"""
A http proxy based on the SocketServer Module
Author: Jonas Wagner
Version: 0.1
Last Change: 2008-06-23

HTTPRipper a generic ripper for the web
Copyright (C) 2008 Jonas Wagner

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import SocketServer
import socket
from urlparse import urlparse
import logging

logger = logging

socket.setdefaulttimeout(30)


class HTTPProxyHandler(SocketServer.StreamRequestHandler):
    """handles a connection from the client, can handle multiple requests"""
    def parse_request(self):
        """parse a request line"""
        request = ""
        while not request:
            request = self.rfile.readline().strip()
        logger.debug("request %r", request)
        method, rawurl, version = request.split(" ")
        return method, rawurl, version

    def parse_header(self, f):
        """read the httpheaders from the file like f into a dictionary"""
        logger.debug("processing headers")
        headers = {}
        for line in f:
            if not line.strip():
                break
            key, value = line.split(": ", 1)
            headers[key] = value.strip()
        return headers

    def write_headers(self, f, headers):
        """
        Forward the dictionary containing httpheaders *headers*
        to the file f. Writes a newline at the end.
        """
        logger.debug("forwarding headers %r", headers)
        for item in headers.items():
            if not item[0].startswith("Proxy-"):
                f.write("%s: %s\r\n" % item)
        f.write("\r\n")

    def forward(self, f1, f2, maxlen=0):
        """forward maxlen bytes from f1 to f2"""
        logger.debug("forwarding %r bytes", maxlen)
        left = maxlen or 1000000000
        while left:
            data = f1.read(min(left, 1024))
            if not data:
                break
            f2.write(data)
            left -= len(data)

    forward_request_body = forward
    forward_response_body = forward

    def request_url(self, method, rawurl, version):
        """create a new socket and write the requestline"""
        url = urlparse(rawurl)
        request = "%s %s%s %s\r\n" % (method, url.path or "/",
                url.query and "?" + url.query or "", version)
        logging.debug("request_url(%r, %r, %r) request: %r", method, rawurl, version, request)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        s.connect((url.hostname, int(url.port or 80)))
        s.sendall(request)
        return s, s.makefile("rwb", 0)

    def __repr__(self):
        return "HTTPProxyRequestHandler(%r)" % self.url

    def handle(self):
        try:
            self._handle()
        except:
            logger.exception("An error occured while handling request %r", self)
            raise

    def _handle(self):
        """Handle client requests"""
        while True:
            method, url, version = self.parse_request()
            self.url = url
            self.requestheaders = self.parse_header(self.rfile)
            self.requestheaders["Connection"] = "close"
            sock, request = self.request_url(method, url, version)
            self.write_headers(request, self.requestheaders)
            if method in ("POST", "PUT") and "Content-Length" in self.requestheaders:
                self.forward_request_body(self.rfile, request,
                        int(self.requestheaders["Content-Length"]))
                sock.shutdown(socket.SHUT_WR)
            # forward status line
            self.wfile.write(request.readline())
            self.responseheaders = self.parse_header(request)
            self.write_headers(self.wfile, self.responseheaders)
            try:
                clen = int(self.responseheaders.get("Content-Length"))
            except (KeyError, TypeError, ValueError):
                clen = None
            self.forward_response_body(request, self.wfile, clen)
            try:
                request.close()
                sock.shutdown(socket.SHUT_RD)
            except:
                pass
            sock.close()
            if self.requestheaders.get("Proxy-Connection") != "keep-alive":
                break
            break

class HTTPProxyServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
    allow_reuse_address = True
    daemon_threads = True
    timeout = 90
    request_queue_size = 10

    def __init__(self, addr):
        SocketServer.TCPServer.__init__(self, addr, HTTPProxyHandler)

    def handle_error(self, request, addr):
        pass

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    server = HTTPProxyServer(("localhost", 8080))
    server.serve_forever()




#!/usr/bin/env python
"""A simple crossplatform autostart helper"""
from __future__ import with_statement

__all__ = "add", "exists", "remove"

import os
import sys

if sys.platform == 'win32':
    import _winreg
    _registry = _winreg.ConnectRegistry(None, _winreg.HKEY_CURRENT_USER)
    def get_runkey():
        return _winreg.OpenKey(_registry,
                r"Software\Microsoft\Windows\CurrentVersion\Run", 0, 
		_winreg.KEY_ALL_ACCESS)

    def add(name, application):
        """add a new autostart entry"""
        key = get_runkey()
        _winreg.SetValueEx(key, name, 0, _winreg.REG_SZ, application)
        _winreg.CloseKey(key)

    def exists(name):
        """check if a autostart entry exists"""
        key = get_runkey()
        exists = True
        try:
            _winreg.QueryValueEx(key, name)
        except WindowsError:
            exists = False
        _winreg.CloseKey(key)
        return exists

    def remove(name):
        """delete a autostart entry"""
        key = get_runkey()
        _winreg.DeleteValue(key, name)
        _winreg.CloseKey(key)
else:
    _xdg_config_home = os.path.expanduser(os.environ.get("XDG_CONFIG_HOME", "~/.config"))
    if not os.path.exists(_xdg_config_home):
        os.mkdir(_xdg_config_home)
    _xdg_user_autostart = os.path.join(_xdg_config_home, "autostart")
    if not os.path.exists(_xdg_user_autostart):
        os.mkdir(_xdg_user_autostart)

    def getfilename(name):
        """get the filename of a autostart (.desktop) file"""
        return os.path.join(_xdg_user_autostart, name + ".desktop")

    def add(name, application):
        """add a new autostart entry"""
        desktop_entry = "[Desktop Entry]\n"\
            "Name=%s\n"\
            "Exec=%s\n"\
            "Type=Application\n"\
            "Terminal=false\n" % (name, application)
        with open(getfilename(name), "w") as f:
            f.write(desktop_entry)

    def exists(name):
        """check if a autostart entry exists"""
        return os.path.exists(getfilename(name))

    def remove(name):
        """delete a autostart entry"""
        os.unlink(getfilename(name))

def test():
    try:
        add("test_xxx", "test")
        assert exists("test_xxx")
    finally:
        remove("test_xxx")

if __name__ == "__main__":
    test()



"""
Author: Jonas Wagner
Version: 0.1
Last Change: 2008-08-20

HTTPRipper a generic ripper for the web
Copyright (C) 2008 Jonas Wagner

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import datetime
import logging
import os
from os import path
import shutil
import socket
import SocketServer
import sys
import tempfile
import threading
import time
from urlparse import urlparse


# configure gettext
import locale
locale.setlocale(locale.LC_ALL, "")

import gettext
try:
    PREFIX = path.abspath(path.join(path.dirname(sys.modules["__main__"].__file__), os.pardir))
except AttributeError:
    # frozen
    PREFIX = path.abspath(".")
LANGUAGES = [locale.getdefaultlocale()[0] or "en_US", "en_US"]
LOCALE_PATH = os.path.join(PREFIX, "share", "locale")
gettext.bindtextdomain('httpripper', LOCALE_PATH)
gettext.textdomain('httpripper')
gettext.translation('httpripper', LOCALE_PATH,
        languages=LANGUAGES, fallback = True).install(unicode=1)


import gtk, gobject, pango

from x29a import mygtk
from x29a.utils import byteformat
mygtk.register_webbrowser_url_hook()

import prox as proxpy

try:
    __file__
except NameError:
    # frozen
    sys.stderr.write = lambda *x: None

NAME = "HTTPRipper"
VERSION = "1.0"
WEBSITE = "http://29a.ch/httpripper/"

def llabel(s):
    """a left aligned label"""
    l = gtk.Label(s)
    l.set_property("xalign", 0.0)
    return l

def byteformatdatafunc(column, cell, model, treeiter):
    n = int(cell.get_property('text'))
    cell.set_property('text', byteformat(n))

def get_unused_filename(name):
    if path.exists(name):
        root, ext = path.splitext(name)
        i = 1
        while path.exists(root + str(i) + ext):
            i += 1
        return root + str(i) + ext
    return name

class ContentTypeFilter(gtk.ComboBox):
    """A combobox to choose a prefix to filter mimetypes"""
    content_types = [
            ("text-x-generic", _("Any"), ""),
            ("audio-x-generic", _("Audio"), "audio"),
            ("image-x-generic", _("Image"), "image"),
            ("video-x-generic", _("Video"), "video"),
    ]
    def __init__(self):
        self.model = mygtk.ListStore(icon=gtk.gdk.Pixbuf, name=str, prefix=str)
        for icon, name, prefix in self.content_types:
            self.model.append(icon=mygtk.iconfactory.get_icon(icon, 16), name=name, prefix=prefix)
        gtk.ComboBox.__init__(self, self.model)
        pixbuf_cell = gtk.CellRendererPixbuf()
        self.pack_start(pixbuf_cell, False)
        self.add_attribute(pixbuf_cell, 'pixbuf', self.model.columns.icon)
        text_cell = gtk.CellRendererText()
        self.pack_start(text_cell, True)
        self.add_attribute(text_cell, 'text', self.model.columns.name)
        self.set_active(0)

    @property
    def prefix(self):
        i = self.get_active()
        return self.model.get_value(self.model.get_iter(self.get_active()),
                self.model. columns.prefix)


class MainWindow(gtk.Window):
    """the main window of httpripper"""
    def __init__(self):
        gtk.Window.__init__(self)
        self.port = 8080
        while True:
            try:
                self.server = HTTPProxyServer(self)
                break
            except socket.error:
                self.port += 1
        self.server.start()
        self.set_title(NAME)
        self.set_icon(mygtk.iconfactory.get_icon("httpripper", 32))
        self.set_default_size(600, 600)
        self.vbox = gtk.VBox()
        self.add(self.vbox)
        self.info = gtk.Label(
                _("HTTPRipper is running on localhost: %i") % self.port)
        self.vbox.pack_start(self.info, False, False)
        self.model = mygtk.ListStore(date=str, url=str, size=int, path=str,
                icon=str, content_type=str)
        self.model_filtered = self.model.filter_new()
        self.model_filtered.set_visible_func(self.row_visible)
        self.model_sort = gtk.TreeModelSort(self.model_filtered)
        self.treeview = gtk.TreeView(self.model_sort)
        self.treeview.set_rules_hint(True)
        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.treeview.connect("row-activated", self.save_file)
        self.treeview.set_search_column(self.model.columns.url)

        # add rows
        col = self.treeview.insert_column_with_attributes(0, _("Time"),
                gtk.CellRendererText(), text=self.model.columns.date)
        col.set_sort_column_id(self.model.columns.date)

        col = self.treeview.insert_column_with_attributes(1, _("Size"),
                gtk.CellRendererText(), text=self.model.columns.size)
        col.set_cell_data_func(col.get_cell_renderers()[0], byteformatdatafunc)
        col.set_sort_column_id(self.model.columns.size)

        renderer = gtk.CellRendererText()
        col = self.treeview.insert_column_with_attributes(2, _("Content-Type"),
                renderer, text=self.model.columns.content_type)
        renderer.set_property("ellipsize", pango.ELLIPSIZE_END)
        renderer.set_property("width", 100)
        col.set_sort_column_id(self.model.columns.content_type)

        col = self.treeview.insert_column_with_attributes(3, _("URL"),
                gtk.CellRendererText(), text=self.model.columns.url)
        col.set_expand(True)
        col.set_sort_column_id(self.model.columns.url)

        col = self.treeview.insert_column_with_attributes(0, "",
                gtk.CellRendererPixbuf(),
                **({"icon_name": self.model.columns.icon}))

        self.vbox.pack_start(mygtk.scrolled(self.treeview))
        #self.treeview.set_fixed_height_mode(True) # makes it a bit faster

        # filtering
        self.filter_content_type = ContentTypeFilter()
        self.filter_size = gtk.Entry()
        self.filter_size.connect("changed", lambda entry: self.model_filtered.refilter())
        self.filter_content_type.connect("changed", lambda combobox: self.model_filtered.refilter())
        self.filter_expander = gtk.Expander(label=_("Filter"))
        self.filter_expander.add(
                mygtk.make_table([
                    (llabel(_("Content-Type")), llabel(_("is")), self.filter_content_type),
                    (llabel(_("Size")), llabel(_("at least (KiB)")), self.filter_size),
                ])
        )
        self.vbox.pack_start(self.filter_expander, False, False)

        # buttons at the buttom
        self.buttonbox = gtk.HButtonBox()
        self.vbox.pack_end(self.buttonbox, False, False)

        self.button_record = gtk.ToggleButton(gtk.STOCK_MEDIA_RECORD)
        self.button_record.set_use_stock(True)
        self.button_record.connect("clicked", self.record)
        self.buttonbox.pack_start(self.button_record)

        self.button_save = gtk.Button(stock=gtk.STOCK_SAVE_AS)
        self.button_save.connect("clicked", self.save)
        self.buttonbox.pack_start(self.button_save)

        self.button_clear = gtk.Button(stock=gtk.STOCK_CLEAR)
        self.button_clear.connect("clicked", self.clear)
        self.buttonbox.pack_start(self.button_clear)

        self.button_about = gtk.Button(stock=gtk.STOCK_ABOUT)
        self.button_about.connect("clicked", self.about)
        self.buttonbox.pack_start(self.button_about)

        self.connect("destroy", self.clear)
        self.connect("destroy", lambda *args: self.server.shutdown())
        self.connect("destroy", gtk.main_quit)

    def save(self, sender):
        """save the selected files"""
        model, rows = self.treeview.get_selection().get_selected_rows()
        if len(rows) == 1:
            self.save_file(self.treeview, rows[0], None)
        elif len(rows) > 1:
            self.save_files(model, rows)

    def save_files(self, model, rows):
        """called to save multiple selected files"""
        dialog = gtk.FileChooserDialog(
                title=_("Save As"),
                parent=self,
                action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                buttons=(# ugly pygtk problem
                    _("Cancel").encode("utf8"), gtk.RESPONSE_CANCEL,
                    _("Save").encode("utf8"), gtk.RESPONSE_OK
                )
        )
        if dialog.run() == gtk.RESPONSE_OK:
            for row in map(model.get_iter, rows):
                filepath = model.get_value(row, self.model.columns.path)
                url = model.get_value(row, self.model.columns.url)
                name = path.basename(url).split("?")[0]
                dest = get_unused_filename(path.join(dialog.get_filename(), name))
                shutil.copy(filepath, dest)
        dialog.destroy()

    def save_file(self, treeview, treepath, view_column):
        """called to save a single file"""
        model = treeview.get_model()
        row = model.get_iter(treepath)
        filepath = model.get_value(row, self.model.columns.path)
        url = model.get_value(row, self.model.columns.url)
        name = urlparse(url).path.split("/")[-1]
        dialog = gtk.FileChooserDialog(
                title=_("Save As"),
                parent=self,
                action=gtk.FILE_CHOOSER_ACTION_SAVE,
                buttons=( # ugly pygtk problem
                    _("Cancel").encode("utf8"), gtk.RESPONSE_CANCEL,
                    _("Save").encode("utf8"), gtk.RESPONSE_OK
                )
        )
        dialog.set_current_name(name)
        if dialog.run() == gtk.RESPONSE_OK:
            shutil.copy(filepath, dialog.get_filename())
        dialog.destroy()

    def row_visible(self, model, iter_):
        content_type = self.model.get_value(iter_, self.model.columns.content_type)
        prefix = self.filter_content_type.prefix
        if prefix:
            if not content_type:
                return False
            if not content_type.startswith(prefix):
                return False
        try:
            size = int(self.filter_size.get_text())*1024
        except ValueError:
            size = 0
        if size and size > self.model.get_value(iter_, self.model.columns.size):
            return False
        return True

    def new_file(self, url, filepath, content_type):
        """called by the proxy where there is a new file"""
        self.model.append(
                date=datetime.datetime.now().time().strftime("%H:%M:%S"),
                url=url,
                size=path.getsize(filepath),
                icon="gtk-save",
                path=filepath,
                content_type=content_type
        )

    def clear(self, sender):
        """clear the model and remove all files"""
        for row in self.model:
            filepath = row[self.model.columns.path]
            os.remove(filepath)
        self.model.clear()
        self.treeview.columns_autosize()

    def record(self, sender):
        self.server.record = not self.server.record

    def about(self, sender):
        """show an about dialog"""
        about = gtk.AboutDialog()
        about.set_transient_for(self)
        about.set_logo(mygtk.iconfactory.get_icon("httpripper", 128))
        about.set_name(NAME)
        about.set_version(VERSION)
#        about.set_comments("")
        about.set_authors(["Jonas Wagner"])
        about.set_translator_credits(_("translator-credits"))
        about.set_copyright("Copyright (c) 2008 Jonas Wagner")
        about.set_website(WEBSITE)
        about.set_website_label(WEBSITE)
        about.set_license("""
Copyright (C) 2008 Jonas Wagner
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
""")
        about.run()
        about.destroy()

class Tee(object):
    """A filelike that writes it's data to two others"""
    def __init__(self, f1, f2):
        self.f1 = f1
        self.f2 = f2

    def write(self, data):
        self.f1.write(data)
        self.f2.write(data)

class HTTPProxyHandler(proxpy.HTTPProxyHandler):
    """handles a single request to the proxy"""
    def forward_response_body(self, f1, f2, contentlength):
        """forwardes the content to the client and (if record is true) logs it"""
        if self.server.record:
            fd, name = tempfile.mkstemp(dir=self.server.tempdir)
            f3 = os.fdopen(fd, "w+b", 0)
            f2 = Tee(f2, f3)
        self.forward(f1, f2, contentlength)
        if self.server.record:
            self.server.on_new_file(self.url, name, self.responseheaders.get("Content-Type"))

class HTTPProxyServer(proxpy.HTTPProxyServer, threading.Thread):
    """accepts client connections, deletes all files on shutdown"""
    def __init__(self, mainwin):
        self.tempdir = tempfile.mkdtemp(prefix="httpripper")
        self.record = False
        threading.Thread.__init__(self)
        self.setDaemon(True)
        self.mainwin = mainwin
        SocketServer.TCPServer.__init__(self, ("127.0.0.1", mainwin.port),
                HTTPProxyHandler)

    def run(self):
        self.serve_forever()

    def shutdown(self):
        shutil.rmtree(self.tempdir)
        self.socket.close()

    def on_new_file(self, url, filepath, content_type):
        gobject.idle_add(self.mainwin.new_file, url, filepath, content_type)

def main():
    """the entry point"""
    level = "DEBUG" in sys.argv and logging.DEBUG or logging.ERROR
    if sys.platform == "win32":
        def release_gil_on_stupid_operating_system():
            """fix for a stupid problem with threads and gtk on windows!"""
            time.sleep(0.001)
            return True
        gobject.timeout_add(25, release_gil_on_stupid_operating_system)
        logging.basicConfig(filename="httpripper.log", level=level)
    else:
        logging.basicConfig(level=level)
        gtk.gdk.threads_init()
    win = MainWindow()
    win.show_all()
    gtk.main()

if __name__ == "__main__":
    main()



import sys

def usage(commands):
    """Show this help message"""
    print __doc__
    print "\nUsage: %s [options] command [arguments...]" % sys.argv[0]
    print "\nCommands:"
    for cmd in commands:
        print (cmd.func_name + " " + " ".join(
                cmd.func_code.co_varnames[:cmd.func_code.co_argcount])).ljust(\
                        25) + (cmd.__doc__ or "")
    print "\nOptions:"
    print "-verbose"

def main(commands):
    verbose = False
    if "-verbose" in sys.argv:
        sys.argv.remove("-verbose")
        verbose = True
    if len(sys.argv) < 2:
        usage(commands)
        return 1
    for cmd in commands:
        if cmd.func_name == sys.argv[1]:
            try:
                cmd(*sys.argv[2:])
                return 0
            except Exception, e:
                if verbose:
                    import traceback
                    traceback.print_exc()
                else:
                    print e.message
                usage(commands)
                return 1
    print "Unknown command"
    usage(commands)
    return 1




import math

class Pluggable(list):
    def __call__(self, f):
        self.append(f)
        return f

def byteformat(n, units=('B', 'KiB', 'MiB', 'GiB', 'TiB')):
    """Format a number of bytes"""
    i = n and int(math.log(n, 1024)) or 0
    if i >= len(units):
        i = len(units)-1
    n /= 1024.0**i
    return "%.2f %s" % (n, units[i])




#!/usr/bin/env python
import sys
import os

from distutils.core import setup

def ls_r(dir):
    def do_reduce(a, b):
        files = []
        for f in b[2]:
            files.append(os.path.join(b[0], f))
        a.append((b[0], files))
        return a
    return reduce(do_reduce, os.walk(dir), [])

kwargs = {
      'name': 'httpripper',
      'version': "1.0.0",
      'description': 'A generic ripper for the web implemented as a http proxy',
      'author': 'Jonas Wagner',
      'author_email': 'veers@gmx.ch',
      'url': 'http://29a.ch/httpripper/',
      'packages': ['x29a', 'httpripper'],
      'scripts': ['bin/httpripper'],
      'options': {'py2exe':{
          'packages': 'encodings',
          'includes': 'cairo, pango, pangocairo, atk, gobject',
          'dist_dir': 'dist/win32',
          'optimize': 2,
          }},
      'data_files': ls_r('share'),
      'license': 'GNU GPL v3',
      'classifiers': ['Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: Microsoft',
        'Programming Language :: Python',
        ]
}

try:
    import py2exe
    kwargs['windows'] = [{'script': 'bin/httpripper',
          'icon_resources': [(1, 'httpripper.ico')],
          'dest_base': 'httpripper'}]
except ImportError:
    pass

setup(**kwargs)



import gtk, gobject
import sys

class IconFactory:
    def __init__(self, icon_theme):
        self.cache = {}
        self.icon_theme = icon_theme

    def guess_icon(self, filename, size):
        return self.get_icon(self.guess_icon_name(filename), size)

    def get_icon(self, name, size):
        # damn it! why did no one tell me I have to cache this!!!
        if not (name, size) in self.cache:
            try:
                self.cache[(name, size)] = self.icon_theme.load_icon(name,
                        size, 0)
            except gobject.GError, e:
                #logger.exception("Unable to load icon %r probably your "
                #        "icon theme isn't conforming to the icon naming "
                #        "convention. You might want to try the tango icon "
                #        "theme", name)
                return None
        return self.cache[(name, size)]

    def get_image(self, name, size):
        pixbuf = self.get_icon(name, size)
        img = gtk.Image()
        img.set_from_pixbuf(pixbuf)
        return img

    def has_icon(self, name):
        return self.icon_theme.has_icon(name)

icon_theme = gtk.icon_theme_get_default()
iconfactory = IconFactory(icon_theme)

def idle_do(func, *args):
    """wrapper arround idle_add that will always run once"""
    def wrapper(*args):
        # throw away the result of the function!
        func(*args)
    gobject.idle_add(wrapper, *args)

def scrolled(widget, shadow=gtk.SHADOW_NONE):
    window = gtk.ScrolledWindow()
    window.set_shadow_type(shadow)
    window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    if any(isinstance(widget, widgettype)
            for widgettype in (gtk.TextView, gtk.Layout)):
        window.add(gtk.Frame(widget))
    else:
        window.add_with_viewport(widget)
    return window

def make_table(widgets):
    """return a gtk.Table containing all the widgets"""
    columns = max(map(len, widgets))
    table = gtk.Table(len(widgets), columns, False)
    for y, row in enumerate(widgets):
        for x, widget in enumerate(row):
            if widget:
                table.attach(widget, x, x+1, y, y+1,
                        xoptions=gtk.EXPAND|gtk.FILL, xpadding=4, ypadding=4)
    return table

def register_webbrowser_url_hook():
    """registers pythons webbrowser module as url_hook"""
    import webbrowser
    def open_url(d, link, data):
        webbrowser.open(link)
    if sys.platform.startswith("linux"):
        webbrowser.register("xdg-open", None,
                webbrowser.GenericBrowser('xdg-open'),  update_tryorder=-1)
    gtk.about_dialog_set_url_hook(open_url, None)

class gtklock:
    """A context manger for the gtk.gdk.threads_*

    can be used like this
    >> with gtklock:
    >>     pass
    """
    @staticmethod
    def __enter__():
        gtk.gdk.threads_enter()

    @staticmethod
    def __exit__(*args):
        gtk.gdk.threads_leave()

def gtk_yield():
    """process all the peding events in the mainloop"""
    while gtk.events_pending():
         gtk.main_iteration()

class IconButton(gtk.Button):
    def __init__(self, icon, size=None, label=None):
        gtk.Button.__init__(self)
        self.size = size or gtk.ICON_SIZE_BUTTON
        self.hbox = gtk.HBox()
        self.add(self.hbox)
        self.img = None
        self.set_icon(icon)
        self.label = None
        self.set_label(label)

    def set_label(self, label):
        if self.label:
            self.label.set_text(label)
        else:
            self.label = gtk.Label(label)
            self.hbox.pack_end(self.label)

    def set_icon(self, icon):
        if self.img:
            self.img.set_from_pixbuf(iconfactory.get_icon(icon, self.size))
        else:
            self.img = iconfactory.get_image(icon, self.size)
            self.hbox.pack_start(self.img)
        self._icon = icon
    icon = property(lambda self: self._icon, set_icon)

class IconMenuItem(gtk.ImageMenuItem):
    icon_size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)[0]
    def __init__(self, icon, text):
        gtk.ImageMenuItem.__init__(self)
        self.set_image(iconfactory.get_image(icon, self.icon_size))
        label = gtk.Label(text)
        label.set_alignment(0.0, 0.5)
        self.add(label)
        self.show_all()

def show_error(msg):
    """Show a 'nice' errbox"""
    dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, message_format=str(msg),
            buttons=gtk.BUTTONS_OK)
    dialog.set_title("Error")
    # dialog.run() - this breaks when called from gobject.idle_add
    # dialog.hide()
    # dialog.destroy
    dialog.show()
    dialog.connect("response", lambda dialog, response: dialog.destroy())

def make_menu(entries, menu):
    for entry in entries:
        if entry is None:
            sub = None
            item = gtk.SeparatorMenuItem()
        else:
            key, sub = entry
            if isinstance(key, tuple):
                item = IconMenuItem(*key)
            elif key.startswith("gtk-"):
                item = gtk.ImageMenuItem(stock_id=key)
            else:
                item = gtk.MenuItem(key)
        if sub:
            if callable(sub):
                item.connect("activate", sub)
            else:
                submenu = gtk.Menu()
                item.set_submenu(submenu)
                make_menu(sub, submenu)
        menu.append(item)

def make_table(widgets):
    """return a gtk.Table containing all the widgets"""
    columns = max(map(len, widgets))
    table = gtk.Table(len(widgets), columns, False)
    for y, row in enumerate(widgets):
        for x, widget in enumerate(row):
            table.attach(widget, x, x+1, y, y+1, xoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=4, ypadding=4)
    return table

class ListStore(gtk.ListStore):
    class Columns(list):
        def __getattr__(self, name):
            try:
                return self.index(name)
            except ValueError:
                raise AttributeError, name

        def ordered(self, valuedict):
            return [valuedict.get(key) for key in self]

    def __init__(self, **kwargs):
        gtk.ListStore.__init__(self, *kwargs.values())
        self.columns = ListStore.Columns(kwargs.keys())

    def serialize(self):
        data = []
        for row in self:
            row_dict = {}
            for i, column in enumerate(self.columns):
                row_dict[column] = row[i]
            data.append(row_dict)
        return data

    def unserialize(self, data):
        for row in data:
            self.append(self.columns.ordered(row))


    def append(self, *args, **kwargs):
        if args:
            gtk.ListStore.append(self, *args)
        else:
            gtk.ListStore.append(self, self.columns.ordered(kwargs))

